import ConfigParser, os, subprocess, sys, libvirt, re, datetime, time, systemd, systemd.journal, check_sender
from xml.dom.minidom import parseString

OK = 0
WARNING = 1
CRITICAL = 2
UNKNOWN = 3

PLUGINS_PATH = '/usr/lib64/nagios/plugins/'

def get_uuid():
	try:
		parser = ConfigParser.ConfigParser(allow_no_value=True)
		parser.read('/etc/c2padmin/node.ini')
		uuid = parser.get('node','uuid')
		return uuid
		
	except:
		return None
		
		
def get_naemon_host():
	NAEMON_HOST = 'naemon-c2p.hosting90.cz'
	ACCEPTOR_PORT = 8180
	try:
		parser = ConfigParser.ConfigParser(allow_no_value=True)
		parser.read('/etc/c2padmin/node.ini')
		host = parser.get('node','naemon_host')
		try:	
			port = parser.get('node','naemon_port')
			return (host, port)
			
		except:
			return (host, 8180)
		
	except:
		return ('naemon-c2p.hosting90.cz', 8180)
		
hostname = get_uuid()
NAEMON_HOST, ACCEPTOR_PORT = get_naemon_host()

def get_sender():
	try:
		sender = check_sender.Check_Sender(NAEMON_HOST, ACCEPTOR_PORT, '/etc/c2padmin/cert/monitoring.key', '/etc/c2padmin/cert/monitoring.crt', '/etc/c2padmin/cert/ca_bundle.crt')
		
	except:
		sys.stderr.write('Couldn\'t connect to monitoring server\n')
		sys.exit(3)
		
	return sender

def check_c2padmin():
	sender = get_sender()
	status = ''

	try:
		status = subprocess.check_output(['/usr/bin/systemctl', 'is-active', 'c2padmin']).strip()
	
	except:
		sender.send_service(hostname, 'c2padmin', 2, 'c2padmin is not running')
		sys.exit(3)
	
	if(status == 'active'):
		sender.send_service(hostname, 'c2padmin', 0, 'c2padmin is running')
		sys.exit(0)
	
	else:
		sender.send_service(hostname, 'c2padmin', 2, 'c2padmin is not running')
		sys.exit(2)
		
def check_libvirt():
	sender = get_sender()

	try:
		conn = libvirt.open()
	
	except:
		sender.send_service(hostname, 'libvirt', 2, 'libvirtd: Failed to connect socket to \'/var/run/libvirt/libvirt-sock\'')
		sys.exit(2)

	not_running = []		
	for domain in conn.listDefinedDomains():
		try:
			dom_object = conn.lookupByName(domain)
		
		except:
			sender.send_service(hostname, 'libvirt', 2, 'libvirtd: Domain not found: no domain with matching name \'virtual_233a\'')
			sys.exit(2)
		
		meta_xml = parseString(dom_object.XMLDesc())
		meta_xml = meta_xml.getElementsByTagName('metadata')[0].getElementsByTagName('c2pauto:metadata')[0]
		autostart = meta_xml.getElementsByTagName('autostart')[0].childNodes[0].data == '1'
		if(autostart):
			not_running.append(domain)

	if(len(not_running) == 0):
		sender.send_service(hostname, 'libvirt', 0, 'libvirtd: OK')
		sys.exit(0)
	
	else:
		sender.send_service(hostname, 'libvirt', 2, 'libvirtd: virtual(s) ' + ', '.join(not_running) + ' not running')
		sys.exit(2)
		
	conn.close()
	
def check_mcelog():
	sender = get_sender()

	status = UNKNOWN
	msg = ''

	seconds = None
	full = False

	file_name = '/var/tmp/.hw_error'
	
	timestamp = datetime.datetime.now() - datetime.timedelta(300)

	# mcelog check
	j = systemd.journal.Reader()
	j.this_boot()
	j.log_level(systemd.journal.LOG_INFO)
	j.add_match(_SYSTEMD_UNIT="mcelog.service")
	j.seek_realtime(timestamp)
	mcelog_data = [(str(x['_SOURCE_REALTIME_TIMESTAMP']),x['MESSAGE']) for x in j]
	j.close()
	# EDAC check
	j = systemd.journal.Reader()
	j.this_boot()
	j.log_level(systemd.journal.LOG_INFO)
	j.add_match(SYSLOG_FACILITY=0)
	j.seek_realtime(timestamp)
	edac_data = [(str(x['__REALTIME_TIMESTAMP']),x['MESSAGE']) for x in j if re.search('EDAC',x['MESSAGE']) != None and re.search('error',x['MESSAGE']) != None]
	j.close()
	# IPMI check
	ipmi_data = []
	cmd ='sudo /usr/bin/ipmitool sel list 2>/dev/null'
	process = subprocess.Popen(cmd, stdin=None, stdout = subprocess.PIPE, stderr=None, shell=True)
	stdout, stderr = process.communicate()
	if process.returncode == 0 and stdout.strip() != '':
		output = stdout.strip().split('\n')
		for line in output:
			record = line.strip().split('|')
			if re.search('Memory',record[3]) != None:
				msg = record[3:]
				event_datetime = datetime.datetime.strptime(" ".join(record[1].strip().split('/')+record[2].strip().split(':')), '%m %d %Y %H %M %S')
				if event_datetime >= timestamp:
					ipmi_data.append([event_datetime," ".join(" ".join(record[3:]).split()),record[0]])

	slot_ids = []
	if len(ipmi_data) > 0:
		cmd = 'dmidecode --type memory'
		process = subprocess.Popen(cmd, stdin=None, stdout = subprocess.PIPE, stderr=None, shell=True)
		stdout, stderr = process.communicate()
		if process.returncode == 0 and stdout.strip() != '': bios_info = stdout.strip()
		for event in [line[-1] for line in ipmi_data]:
			cmd ='sudo /usr/bin/ipmitool sel get %s 2>/dev/null' % int(event,16)
			process = subprocess.Popen(cmd, stdin=None, stdout = subprocess.PIPE, stderr=None, shell=True)
			stdout, stderr = process.communicate()
			if process.returncode == 0 and stdout.strip() != '':
				try:
					event_id = filter(lambda line: re.search(r'Event Data',line),stdout.split('\n'))[0].split(':')[1].strip()
					if int(event_id[:2],16) == 0xa1: ecc = 'uncorrectable ECC'
					elif int(event_id[:2],16) == 0xa0: ecc = 'correctable ECC'
					channel = (int(event_id[2:4],16) >> 4) - 1
					slot = int(event_id[2:4],16) - ((channel +1) << 4) - 0x0a
					cpu = int(event_id[4:],16) - 0x80
					bios_slot_info = filter(lambda x: re.search(re.compile('Channel%s_Dimm%s' % (channel,slot)),x), bios_info.strip().split('\n\n'))[0]
					slot_ids.append(filter(lambda x: '\tLocator' in x, bios_slot_info.split('\n'))[0].split(':')[1].strip())
				except:
					pass

	data = mcelog_data + edac_data + ipmi_data

	f = open('/proc/uptime','r')
	uptime = datetime.timedelta(seconds = float(f.readline().split()[0]))
	f.close()

	file_found = os.path.exists(file_name)
	if file_found:
		file_mtime = time.ctime(os.path.getmtime(file_name))
		file_age = now - datetime.datetime.strptime(file_mtime, "%a %b %d %H:%M:%S %Y")

	if len(data) > 0:
		if len(slot_ids) > 0: 
			msg = "Possible hardware error was detected: %s" % ", ".join(list(set(slot_ids)))
		
		else: 
			msg = "Possible hardware error was detected."
		
		status = CRITICAL
		# vytvorime soubor
		if (not file_found or ( file_found and uptime < file_age )): 
			try:
				os.utime(file_name, None)
				
			except:
				open(file_name, 'a').close()
			
		if len(slot_ids) > 0:
			slot_ids = list(set(slot_ids))
			if not file_found:
				with open(file_name,'w') as f: f.write("\n".join(slot_ids)+"\n")
				
			else:
				with open(file_name,'r') as r: r_content = r.read()
				for slot_id in slot_ids:
					if not (slot_id in r_content.split('\n')):
						with open(file_name,'a') as a: a.write(slot_id+"\n")
						
	else:
		if file_found:
			if uptime > file_age: 
				with open(file_name,'r') as r: r_content = r.read()
				ipmi_slots = r_content.strip().split('\n')
				if len(ipmi_slots) > 0: 
					msg = "Older hardware error was found: %s" % ", ".join(ipmi_slots)
				
				else: 
					msg = "Older hardware error was found."
				
				status = CRITICAL
			
			else:
				os.remove(file_name)
				msg = "No hardware error found."
				status = OK
		else:
			msg = "No hardware error found."
			status = OK

	sender.send_service(hostname, 'mcelog', status, msg)
	sys.exit(status)
	
def host_alive():
	sender = get_sender()
	sender.send_host(hostname, 0, 'HOST ok')
	
def plugin_wrapper(plugin, args = []):
	sender = get_sender()

	if(not os.path.exists(os.path.join(PLUGINS_PATH, 'check_' + plugin))):
		sender.send_service(hostname, plugin, 3, 'plugin not found')
		print 'plugin not found'
		sys.exit(3)
	
	else:
		p = subprocess.Popen([os.path.join(PLUGINS_PATH, 'check_' + plugin)] + args, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
		ret = p.communicate()
		sender.send_service(hostname, plugin, p.returncode, ret[0])
		sys.exit(p.returncode)
		